<?php

namespace Drupal\monarch_bootstrap_layout\Plugin\Layout;

use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configurable vertical tabs layout.
 *
 * @Layout(
 *   id = "bootstrap_tabs",
 *   label = @Translation("Bootstrap Tabs"),
 *   category = @Translation("Bootstrap"),
 *   template = "layout--bootstrap-tabs",
 *   library = "monarch_bootstrap_layout/bootstrap_tabs",
 *   path = "layouts/bootstrap_tabs",
 *   regions = { }
 * )
 */
class BootstrapTabsLayout extends LayoutDefault implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    // Since we're modifying the actual plugin definition, we clone it so it
    // won't affect other instances of this plugin.
    $plugin_definition = clone $plugin_definition;
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $regions = [];

    foreach (array_filter(explode("\n", $configuration['tab_labels'] ?? '')) as $index => $region_entry) {
      if (trim($region_entry)) {
        $region_criteria = array_map('trim', preg_split('/\|+/', $region_entry));
        $region_label = trim(array_pop($region_criteria)) ?: 'NO LABEL';
        $regions['tab' . $index]['criteria'] = array_map(function ($criteria) {
          return array_map('trim', preg_split('/&+/', $criteria));
        }, $region_criteria);
        $regions['tab' . $index]['label'] = $region_label;
      }
    }

    $plugin_definition->setRegions($regions);

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'tab_style' => 'horizontal_tabs',
      'tab_labels' => "First Tab\nSecond Tab\nThird Tab",
      'validate_user' => FALSE,
      'layout_wrapper_class' => 'my-3',
      'nav_wrapper_class' => '',
      'tab_content_wrapper_class' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['tab_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Tab Style'),
      '#options' => [
        'horizontal_tabs' => $this->t('Horizontal Tabs'),
        'horizontal_tabs_justified' => $this->t('Horizontal Tabs (justified)'),
        'horizontal_tabs_fill' => $this->t('Horizontal Tabs (fill)'),
        'horizontal_pills' => $this->t('Horizontal Pills'),
        'horizontal_pills_justified' => $this->t('Horizontal Pills (justified)'),
        'horizontal_pills_fill' => $this->t('Horizontal Pills (fill)'),
        'vertical_pills' => $this->t('Vertical Pills'),
      ],
      '#default_value' => $this->configuration['tab_style'],
    ];

    $form['validate_user'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Validate Route User'),
      '#description' => $this->t('If a user argument exists in the route, it must match the current user.'),
      '#default_value' => $this->configuration['validate_user'],
    ];

    $form['tab_labels'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Tabs'),
      '#description' => $this->t('Enter each tab label on a new line.<br><br><details style="background-color:#505050;"><summary style="background-color:#FFFFFF10;cursor:pointer;user-select:none">Advanced Tab Control...</summary>@line1<br>@line2<br>@line3<br>@line4<br>@line5<br>@line6<br>@line7</details>', [
        '@line1' => $this->t('<u>Require Permission</u><br>&emsp;permission name|Tab Name'),
        '@line2' => $this->t('<u>Require Role</u><br>&emsp;role:role_name|Tab Name'),
        '@line3' => $this->t('<u>Require User</u><br>&emsp;user:123|Tab Name'),
        '@line4' => $this->t('<u>Require Current User</u><br>&emsp;user:current|Tab Name'),
        '@line5' => $this->t('<u>Require a or b or c or etc</u><br>&emsp;a|b|c|etc|Tab Name'),
        '@line6' => $this->t('<u>Require a and b and c and etc</u><br>&emsp;a&b&c&etc|Tab Name'),
        '@line7' => $this->t('<u>Require a and b OR c and etc</u><br>&emsp;a&b|c&etc|Tab Name'),
      ]),
      '#default_value' => $this->configuration['tab_labels'],
    ];

    $form['layout_wrapper_class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS Classes'),
      '#default_value' => $this->configuration['layout_wrapper_class'],
    ];

    $form['nav_wrapper_class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Tab Nav CSS Classes'),
      '#default_value' => $this->configuration['nav_wrapper_class'],
    ];

    $form['tab_content_wrapper_class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Tab Content CSS Classes'),
      '#default_value' => $this->configuration['tab_content_wrapper_class'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['tab_style'] = $form_state->getValue('tab_style');
    $this->configuration['tab_labels'] = $form_state->getValue('tab_labels');
    $this->configuration['validate_user'] = $form_state->getValue('validate_user');
    $this->configuration['layout_wrapper_class'] = $form_state->getValue('layout_wrapper_class');
    $this->configuration['nav_wrapper_class'] = $form_state->getValue('nav_wrapper_class');
    $this->configuration['tab_content_wrapper_class'] = $form_state->getValue('tab_content_wrapper_class');
  }

}
